/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.userfundchangerecord.param;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Getter;
import lombok.Setter;

import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 用户账户资金变动记录编辑参数
 *
 * @author wulei
 * @date  2025/02/07 11:26
 **/
@Getter
@Setter
public class BizUserFundChangeRecordEditParam {

    /** 账户金额变动记录id */
    @Schema(description = "账户金额变动记录id", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "id不能为空")
    private String id;

    /** 用户id */
    @Schema(description = "用户id")
    private String userId;

    /** 变动类别： 1. 主动充值 2.主动消费 3.代金券生效 4.赠送 5.扣减 */
    @Schema(description = "变动类别： 1. 主动充值 2.主动消费 3.代金券生效 4.赠送 5.扣减")
    private String changeType;

    /** 原账户余额 */
    @Schema(description = "原账户余额")
    private BigDecimal oldAccountBalance;

    /** 原代金券金额 */
    @Schema(description = "原代金券金额")
    private BigDecimal oldVoucherBalance;

    /** 现账户余额 */
    @Schema(description = "现账户余额")
    private BigDecimal newAccountBalance;

    /** 现代金券金额 */
    @Schema(description = "现代金券金额")
    private BigDecimal newVoucherBalance;

}
